<?php

namespace Drupal\advanced_help_hint;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * The Hint class produces a hint string for hook_help().
 */
class Hint {

  use StringTranslationTrait;

  /**
   * Module Handler Service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The advanced help plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface|null
   */
  private $advancedHelp;

  /**
   * Constructor for Captcha Service helper.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module Handler Service.
   * @param \Drupal\Component\Plugin\PluginManagerInterface|null $advanced_help
   *   Advance help service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, PluginManagerInterface $advanced_help = NULL) {
    $this->moduleHandler = $module_handler;
    $this->advancedHelp = $advanced_help;
  }

  /**
   * Assemples and returns the hint.
   */
  public function produceHint($project, $doclink = NULL, $repo = FALSE) {
    $output = '';
    if ($doclink) {
      $options = ['attributes' => ['title' => $this->t('Link to documentation of the project online.')]];
      $link = Link::fromTextAndUrl($this->t('online documentation'), Url::fromUri($doclink, $options))->toString();
      $ss = new FormattableMarkup(
        $this->t('Please see the @url.'), ['@url' => $link]
      );
      $output .= $ss;
    }
    if ($this->moduleHandler->moduleExists('advanced_help')) {
      $plugin_manager = $this->advancedHelp;
      $topics = $plugin_manager->getTopics();
      if (!empty($output)) {
        $output = $output . ' ';
      }
      if (!$repo) {
        $output .=
          $this->t("No additional help available for “!project”.",
            [
              '!project' => $project,
            ]
          );
        // Nothing more to do.
        return $output;
      }
      if (!isset($topics[$project])) {
        $output .=
          $this->t("Expected to find some help for project “@project” in the project's help-directory, but could not locate any.",
            [
              '@project' => $project,
            ]
          );
        // Nothing more to do.
        return $output;
      }

      $file = key($topics[$project]);
      $ext = pathinfo($file, PATHINFO_EXTENSION);
      $helpPath = '/admin/help/ah/' . $project;
      if (empty($ext)) {
        $options = ['attributes' => ['title' => $this->t('Link to advanced help')]];
        $link = Link::fromTextAndUrl(
          $this->t('advanced help framework'),
          Url::fromUri('base:' . $helpPath, $options)
        )->toString();
        $ss = new FormattableMarkup(
          $this->t('Additional help is provided by the @help.'), ['@help' => $link]
        );
        $output .= $ss;
      }
      else {
        $options = [
          'attributes' => ['title' => $this->t('Link to included documentation.')],
        ];
        $link = Link::fromTextAndUrl(
          $this->t('README'),
          Url::fromUri('base:' . $helpPath, $options)
        )->toString();
        $ss = new FormattableMarkup(
          $this->t("Additional help is available in the project's “@readme” file."), ['@readme' => $link]
        );
        $output .= $ss;
      }

    }
    elseif ($repo) {
      if (!empty($output)) {
        $output = $output . ' ';
      }
      // Since advanced_help is not enabled we can't check to make sure
      // help is available.
      $options = ['attributes' => ['title' => $this->t('Link to the Advanced Help project.')]];
      $link = Link::fromTextAndUrl($this->t('Advanced Help'), Url::fromUri('https://www.drupal.org/project/advanced_help', $options))->toString();
      $ss = new FormattableMarkup(
        $this->t('If you install the @advhelp module, additional help may be available.'), ['@advhelp' => $link]
          );
      $output .= $ss;
    }
    return $output;
  }

}
